/*============================================================================
  File:     AnalyzingQueryHash.sql

  Summary:  This query helps you to determine how much of your cache is used
			by plans that have the same query hash but have not been parameterized. 
			This shows query_hash and query_plan_hash.
			
  Date:     March 2011

  SQL Server Version: 2005/2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE Credit
go

SET STATISTICS IO ON
go
-- Turn Graphical Showplan ON (Ctrl+K)

-- Update a row to later search on...
UPDATE dbo.member
	SET lastname = 'Tripp'
		WHERE member_no = 1234
go
-- Review the indexes to see if there's an index for LastName
EXEC sp_helpindex 'dbo.member'
go

-- Create an index on LastName ALONE
CREATE INDEX MemberLastName ON dbo.member(LastName)
go

ALTER PROCEDURE dbo.GetMemberInfo
(
	@LastName	varchar(30)
)
AS
SELECT m.* 
FROM dbo.Member AS m
WHERE m.LastName LIKE @LastName
go

-- Execute the proc and it should use the index created above
EXEC GetMemberInfo '%T%' with recompile
go

-- Look for cached obj type: compiled plan, objtype: proc
SELECT cp.objtype, cp.*, st.text
FROM sys.dm_exec_cached_plans AS cp
	CROSS APPLY sys.dm_exec_sql_text(cp.plan_handle) AS st
WHERE st.text LIKE '%GetMemberInfo%'
go

-- Now, dig in a bit deeper to get the query plan to freeze
SELECT st.text, qs.plan_handle, qs.statement_start_offset, p.query_plan 
FROM sys.dm_exec_query_stats AS qs 
	CROSS APPLY sys.dm_exec_sql_text(sql_handle) st
	CROSS APPLY sys.dm_exec_query_plan(plan_handle) p
WHERE st.text LIKE '%GetMemberInfo%'
ORDER BY 1, qs.EXECUTION_COUNT DESC;
go

-- If that's the plan you want to freeze then you need the @plan_handle and the @statement_start_offset
EXECUTE sp_create_plan_guide_from_handle 
    @name =  N'Guide_For_GetMemberInfo',
    @plan_handle = 0x05000500F2A6E25BB8E0250D000000000000000000000000,
    @statement_start_offset = 140;
go

SELECT * FROM sys.plan_guides
go

sp_control_plan_guide N'enable',N'Guide_For_GetMemberInfo'
go

sp_recompile 'GetMemberInfo' -- doesn't really affect the plan guide 
							 -- *Except* that the showplan percentages seem way off 
							 -- They *stay* based on the first person that executes after 
							 -- a recompile... unless you recompile.
go

EXEC GetMemberInfo 'T%'	
	-- still uses the plan guide but changes the percentages to 1/99 (instead of 40/60)
go

EXEC GetMemberInfo 'Tripp'	
	-- still shows 1/99 even though this is really a 40/60 case.
go

EXEC GetMemberInfo 'Tripp' WITH RECOMPILE	
	-- still uses the plan guide but updates the percentages
go

EXEC GetMemberInfo 'Tripp'	
	-- still shows 1/99, goes back to the plan in cache's values.
go

sp_control_plan_guide N'drop',N'Guide_For_GetMemberInfo'
go

-- Finally, from BOL - you can create plans from the XML showplan directly:
USE AdventureWorks;
GO

SELECT City, StateProvinceID, PostalCode FROM Person.Address ORDER BY PostalCode DESC;
GO

DECLARE @xml_showplan nvarchar(max);
SET @xml_showplan = 
	(SELECT query_plan
    FROM sys.dm_exec_query_stats AS qs 
		CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) AS st
		CROSS APPLY sys.dm_exec_text_query_plan(qs.plan_handle, DEFAULT, DEFAULT) AS qp
    WHERE st.text LIKE N'SELECT City, StateProvinceID, PostalCode FROM Person.Address ORDER BY PostalCode DESC;%');

EXEC sp_create_plan_guide 
    @name = N'Guide1_from_XML_showplan', 
    @stmt = N'SELECT City, StateProvinceID, PostalCode FROM Person.Address ORDER BY PostalCode DESC;', 
    @type = N'SQL',
    @module_or_batch = NULL, 
    @params = NULL, 
    @hints =@xml_showplan;
GO

SELECT * FROM sys.plan_guides
go

sp_control_plan_guide N'drop',N'Guide1_from_XML_showplan'
go